import platform
import subprocess
from enum import auto, Enum
from pathlib import Path
from typing import List

import click
import rich
from rich.markdown import Markdown

import gnote


class Opt(Enum):
    default = auto()
    version = auto()
    list = auto()
    edit = auto()
    test = auto()


def show_version():
    import toml
    cfg = toml.load("./pyproject.toml")
    print(cfg['tool']['poetry']['version'])


def show_docs():
    """
    查看文档列表
    """
    paths = __list_docs()
    for doc in paths:
        print(doc.name.replace('.md', ''))


def __list_docs():
    repo_path = gnote.repo_dir
    paths: List[Path] = []

    def list_file(p: Path):
        for sub in p.iterdir():
            if sub.name.startswith('.'):
                continue
            if sub.is_dir():
                list_file(sub)
            else:
                paths.append(sub)

    list_file(repo_path)
    return paths


def __call_pyvim(doc_path):
    cmd = ['powershell'] if 'windows' == platform.system().lower() else []
    subprocess.call(cmd + ['pyvim', doc_path])


def __check_and_sync():
    g = gnote.git_repo
    if g.is_dirty(untracked_files=True):
        g.git.add('.')
        g.git.commit(m='update')
        g.git.push()


def edit_doc(doc_name):
    """
    编辑文件
    """
    docs = __list_docs()
    dct_doc = {p.name.replace('.md', ''): p for p in docs}
    if doc_name in dct_doc.keys():
        __call_pyvim(dct_doc.get(doc_name))
    else:
        new_doc = Path(gnote.repo_dir).joinpath(Path(doc_name).with_suffix('.md'))
        __call_pyvim(new_doc)
    __check_and_sync()
    pass


def show(doc_name):
    docs = __list_docs()
    dct_doc = {p.name.replace('.md', ''): p for p in docs}
    if (p := dct_doc.get(doc_name, None)) is not None:
        content = Path(p).read_text()
        rich.print(Markdown(content))
    else:
        print("not %s doc" % doc_name)


@click.command()
@click.argument('words', nargs=-1, required=False)
@click.option('-v', '--version', 'opt', flag_value=Opt.version)
@click.option('-l', '--list', 'opt', flag_value=Opt.list)
@click.option('-e', '--edit', 'opt', flag_value=Opt.edit)
# @click.option('-t', '--test', 'opt', flag_value=Opt.test)
def entry(words, opt):
    if opt is not None:
        if opt == Opt.version:
            show_version()
        if opt == Opt.list:
            show_docs()
        if opt == Opt.edit and words[0] != '':
            edit_doc(words[0])
        # if opt == Opt.test:
        #     __check_and_sync()
        return
    else:
        text = ' '.join(words)
        show(text)



from pathlib import Path
import git

config_dir = Path.home() / '.gnote'
config_dir.mkdir(parents=True, exist_ok=True)

repo_dir = config_dir / 'repo'
repo_url = 'git@gitee.com:gitgj/code-notes.git'


def __init_repo():
    if (repo_dir / '.git').exists():  # 当本地仓库存在时，拉取更新
        git_local = git.Repo(repo_dir)
        git_remote = git_local.remote()
        git_remote.pull()
    else:  # 当本地仓库不存在时，clone repository
        git_local = git.Repo.clone_from(url=repo_url, to_path=repo_dir, multi_options=['--depth 1'])
        git_local.config_writer().set_value('user', 'name', 'guojian').release()
        git_local.config_writer().set_value('user', 'email', 'guojian_k@qq.com').release()
    return git_local


git_repo = __init_repo()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gnote']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'configparser>=5.0.0,<6.0.0',
 'gitpython>=3.1.7,<4.0.0',
 'loguru>=0.5.1,<0.6.0',
 'pyvim>=3.0.2,<4.0.0',
 'rich==5.1.2',
 'toml>=0.10.1,<0.11.0']

entry_points = \
{'console_scripts': ['gnote = gnote.cli:entry']}

setup_kwargs = {
    'name': 'gnote',
    'version': '0.3.0',
    'description': 'code note book',
    'long_description': None,
    'author': 'guojian',
    'author_email': 'guojian_k@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
